#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>
#include <bluetooth/hci.h>
#include "bdremote.h"

static uint8_t btsnoop_id[] = { 0x62, 0x74, 0x73, 0x6e, 0x6f, 0x6f, 0x70, 0x00 };

void *bdremote_int_listen(struct bdremote *bdremote);

void bdremote_log(char *str)
{
	printf("%s\n", str);
}

int bdremote_connect(struct bdremote *bdremote, bdaddr_t bdaddr,
                    bdremote_mesg_callback_t mesg_callback) {
	struct sockaddr_l2 ctl_remote_addr, int_remote_addr;

	if (bdremote->state == BDREMOTE_CONN) {
		errno = EALREADY;
		bdremote_log("Connection attempted while already established");
		return -1;
	}

	bdremote->mesg_callback = mesg_callback;

	memset(&int_remote_addr, 0, sizeof(int_remote_addr));
	int_remote_addr.l2_family = AF_BLUETOOTH;
	int_remote_addr.l2_bdaddr = bdaddr;
	int_remote_addr.l2_psm = htobs(INT_PORT);

	bdremote->int_socket = socket(AF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);

	if (connect(bdremote->int_socket, (struct sockaddr *)&int_remote_addr,
		             sizeof(int_remote_addr))) {
		bdremote->state = BDREMOTE_ERR;
		bdremote_log("Error establishing interrupt channel connection");
		return -1;
	}

	if (pthread_create(&bdremote->int_thread, NULL,
	                   (void *(*)(void *))&bdremote_int_listen, bdremote)) {
		close(bdremote->int_socket);
		bdremote->state = BDREMOTE_ERR;
		bdremote_log("Error creating interrupt channel listener thread");
		return -1;
	}

	bdremote->state = BDREMOTE_CONN;
	return 0;
}

int bdremote_disconnect(struct bdremote *bdremote)
{
	if (bdremote->state != BDREMOTE_CONN) {
		errno = ENOTCONN;
		bdremote_log("Disconnect attempted on unconnected bdremote");
		return -1;
	}

	pthread_cancel(bdremote->int_thread);
	pthread_join(bdremote->int_thread, NULL);

	close(bdremote->int_socket);

	bdremote->state = BDREMOTE_NOCONN;

	return 0;
}

#define READ_BUF_LEN 17
void *bdremote_int_listen(struct bdremote *bdremote)
{
	unsigned char buf[READ_BUF_LEN];
	int len;

	do {
		if ((len = read(bdremote->int_socket, buf, 1) == -1)) {
			bdremote_log("Interrupt channel read error");
		}
		else {
			bdremote->mesg_callback(BDREMOTE_BUTTON, 1, buf);
		}
	} while (-1);
	return NULL;
}

