#include <stdio.h>					/* printf */
#include <unistd.h>					/* sleep */
#include <sys/select.h>				/* select */
#include <bluetooth/bluetooth.h>	/* bdaddr_t */
#include <bdremote.h>

#define MENU 				\
	"1: Exit\n"

#define PROMPT "Enter Selection:"

#define MENU_EXIT		1

struct bdremote bdremote;	/* bdremote handle */

int readint(int *input, int base);
int process_user_input();
bdremote_mesg_callback_t bdremote_callback;

int main(int argc, char *argv[])
{
	bdaddr_t bdaddr;		/* bdremote bdaddr */
	fd_set rfds;			/* select read fd set */
	int quit = 0;			/* main loop quit flag */

	/* Get BDRemote address */
	if (argc > 1) {
		if ((strcmp(argv[1], "-h") == 0)) {
			printf("usage:%s [BDADDR]\n", argv[0]);
			return -1;
		}
		else {
			str2ba(argv[1], &bdaddr);
		}
	}

	/* Connect to the BDRemote */
	printf("Connection to remote established.\n");
	printf("Press Start+Enter to make your remote discoverable.\n");
	if (bdremote_connect(&bdremote, bdaddr, &bdremote_callback)) {
		printf("unable to connect\n");
		return 0;
	}

	printf("%s", MENU);
	/* Main Loop */
	do {
		printf("\n%s", PROMPT);
		fflush(stdout);
		quit = process_user_input();
	} while (!quit);

	/* Disconnect */
	if (bdremote_disconnect(&bdremote)) {
		printf("error on disconnect\n");
	}

	/* End */
	return 0;
}

#define BUFSIZE 256
int readint(int *input, int base)
{
	static char buf[BUFSIZE];
	char *str;	/* strtol endptr */

	/* Check for errors */
	if (!fgets(buf, BUFSIZE, stdin)) {
		return -1;
	}
	/* Check for buffer overflow */
	else if (buf[strlen(buf)-1] != '\n') {
		return -1;
	}

	*input = strtol(buf, &str, base);
	if (*str != '\n') {
		return -1;
	}
	return 0;
}

int process_user_input()
{
	int input;
	int quit = 0;

	if (readint(&input, 10)) {
		printf("input error\n");
		quit = -1;
	}
	else {
		switch (input) {
		case MENU_EXIT:
			quit = -1;
			break;
		default:
			printf("Invalid selection\n");
			break;
		}
	}

	return quit;
}

void bdremote_callback(enum bdremote_mesg mesg, int len, unsigned char *data)
{
	switch (mesg) {
	case BDREMOTE_BUTTON:
		printf("\nButton status: %x%x%x%x%x%x%x%x%x%x%x%x%x\n%s", data[0], data[1], PROMPT);
		fflush(stdout);
		break;
	default:
		break;
	}
}

